<?php
/*
Plugin Name: wp-forecast
Plugin URI: http://www.tuxlog.de
Description: wp-forecast is a plugin for wordpress, 
             showing weather-data from accuweather.com.
Version: 0.1
Author: Hans Matzen <webmaster at tuxlog.de>
Author URI: http://www.tuxlog.de
*/

/*  Copyright 2006  Hans Matzen  (email : webmaster at tuxlog.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// include translations
require_once("language.php");
// generic functions
require_once("functions.php");
// include admin options page
require_once("wp-forecast-admin.php");

// get options
$location=get_option("wp-forecast-location");
$refresh=get_option("wp-forecast-refresh"); 
$metric=get_option("wp-forecast-metric"); 
$language=get_option("wp-forecast-language");
$days=get_option("wp-forecast-days");

$BASE_URI="http://forecastfox.accuweather.com/adcbin/forecastfox/weather_data.asp?";

// fill language array
$tl=array();
$tl=set_translation($language);

// these are the xml-paths we wish to fetch from the weather-data
$path_tabelle = 
array(
      "/ADC_DATABASE/UNITS/TEMP"  => "un_temp",
      "/ADC_DATABASE/UNITS/DIST"  => "un_dist",
      "/ADC_DATABASE/UNITS/SPEED" => "un_speed",
      "/ADC_DATABASE/UNITS/PRES"  => "un_pres",
      "/ADC_DATABASE/UNITS/PREC"  => "un_prec",
      "/ADC_DATABASE/LOCAL/CITY"  => "city",
      "/ADC_DATABASE/LOCAL/STATE" => "state",
      "/ADC_DATABASE/LOCAL/TIME"  => "time",
      "/ADC_DATABASE/CURRENTCONDITIONS/PRESSURE"  => "pressure",
      "/ADC_DATABASE/CURRENTCONDITIONS/TEMPERATURE"  => "temperature",
      "/ADC_DATABASE/CURRENTCONDITIONS/REALFEEL"  => "realfeel",
      "/ADC_DATABASE/CURRENTCONDITIONS/HUMIDITY"  => "humidity",
      "/ADC_DATABASE/CURRENTCONDITIONS/WEATHERTEXT"  => "weathertext",
      "/ADC_DATABASE/CURRENTCONDITIONS/WEATHERICON"  => "weathericon",
      "/ADC_DATABASE/CURRENTCONDITIONS/WINDSPEED"  => "windspeed",
      "/ADC_DATABASE/CURRENTCONDITIONS/WINDDIRECTION"  => "winddirection"
);
$weather=array();


//
// build the url from the parameters and fetch the weather-data
// return it as one long string
//
function get_weather($uri,$loc,$metric)
{
  $url=$uri . "location=" . urlencode($loc) . "&metric=" . 
    $metric . "&partner=forecastfox";
  $xml = fetchURL($url);

  return $xml;
}


//
// parses the xml for the paths in path_tabelle and store the values
// a cookie
//
function store_weather($weather_xml,$ref)
{
  global $path_tabelle,$weather;
  $pstack="";
  $go_ahead = "";

  // start_element() - wird vom XML-Parser bei öffnenden Tags aufgerufen
  function start_element( $parser, $name, $attribute )
    {
      // Umwandlungstabelle in den lokalen Scope holen
      global $go_ahead,$pstack,$path_tabelle;
      $pstack=$pstack."/$name";

      if (isset( $path_tabelle[ $pstack]))
	$go_ahead = $path_tabelle[$pstack];
    }
  
  // end_element() - wird vom XML-Parser bei schließenden Tags aufgerufen
  function end_element( $parser, $name )
    {
      // pfad abbauen
      global $pstack;
      $pstack = substr($pstack,0, strrpos($pstack,"/"));
    }
  
  // daten() - wird vom XML-Parser für CDATA-Bereiche aufgerufen 
  function daten( $parser, $data )
    {
      global $go_ahead, $weather;
      
      if ( strlen($go_ahead) > 0)
	{
	  $weather[$go_ahead]=$data;
	  $go_ahead = '';
	}
    }

  // Instanz des XML-Parsers erzeugen
  $parser = xml_parser_create();
  
  // Parameter des XML-Parsers setzen 
  xml_parser_set_option( $parser, XML_OPTION_CASE_FOLDING, true ); 

  // Handler für Elemente ( öffnende / schließende Tags ) setzen 
  xml_set_element_handler( $parser, "start_element", "end_element" ); 
  // Handler für Daten ( CDATA ) setzen
  xml_set_character_data_handler( $parser, "daten" );

  // Versuche zu parsen
  if( !xml_parse( $parser, $weather_xml,true ) )
    {
      // Fehler -> Ausführung abbrechen
      die(  "XML Fehler: " . 
	    xml_error_string( xml_get_error_code( $parser ) ) . 
	    " in Zeile " .
	    xml_get_current_line_number( $parser )
	    );
    }

  // Vom XML-Parser belegten Speicher freigeben
  xml_parser_free( $parser );
  
  // ergebnis als cookie speichern
  setcookie ("wp_forecast_cache", arr2str($weather),time()+$ref,"/wordpress/");
  // und zurueckgeben
  return $weather;
}


//
// display the weather-data 
//
function show($w)
{
    global $tl;
    $plugin_path = get_settings('siteurl') . '/wp-content/plugins/wp-forecast';
    $out ="";
    $out .="\n<div class=\"wp-forecast\">\n";
    $out .="<center><b>".$tl['title']."</b></center>\n";
    $out .= $tl['loc'].": ".$w["city"]." ".$w["state"]."<br />\n";
    $out .="<table class=\"wp-forecast\">\n";
    $out .= "<tr><td><img src='".$plugin_path."/icons/".$w["weathericon"].".gif' alt='".$tl[$w["weathericon"]]."'></td>\n";
    $out .= "<td>".$tl['time']." ". $w["time"]."<br />";
    $out .= $tl[$w["weathericon"]]."<br />";
    $out .= $tl['tmp'].": ".$w["temperature"]."&deg;";
    $out .= $w['un_temp']."</td></tr></table>\n";
    $out .= $tl['flik'].": ".$w["realfeel"]."&deg;".$w['un_temp']."<br />\n";
    $out .= $tl['barr'].": ".$w["pressure"]." ".$w["un_pres"]."<br />\n";
    $out .= $tl['hmid'].": ".$w["humidity"]."<br />\n";
    $out .= $tl['winds'].": ".$w["windspeed"]." ".$w["un_speed"]." " . $w["winddirection"]."<br />\n";
    $out .="</div><br />\n";
    echo $out;
}

//
// just return the css link
// this function is called via the wp_head hook
//
function wp_forecast_css() {
  $plugin_path = get_settings('siteurl') . '/wp-content/plugins/wp-forecast';
  echo "<link rel=\"stylesheet\" href=\"". $plugin_path. "/wp-forecast.css\" type=\"text/css\" media=\"screen\" />\n";
}

//
// set cookie with weather data for current parameters
// a wrapper function called via the init hook
//
function wp_forecast_init() 
{
    global $location,$refresh,$metric,$days,$language;
    global $BASE_URI,$path_tabelle,$weather;

    if (! isset($_COOKIE["wp_forecast_cache"] ))
    {
	$w = get_weather($BASE_URI,$location,$metric);
	$weather=store_weather($w,$refresh);
    }
}

// 
// this function is called from your template
// to insert your weather data at the place you wannt it to be
//
function wp_forecast()
{
    global $weather;
    if (isset ($_COOKIE["wp_forecast_cache"]) )
	$weather=str2arr($_COOKIE["wp_forecast_cache"]);
    show($weather);
}

//
// map the various functions to the hooks
//

// add activate/deactivate to not mess up the database
add_action('activate_wp-forecast/setup.php','activate');
add_action('deactivate_wp-forecast/setup.php','deactivate');

// add option page 
add_action('admin_menu', 'wp_forecast_admin');

// add fetch weather data to init the cookie before any headers are sent
add_action('init','wp_forecast_init');
add_action('init','wp_forecast_admin_init');

// add css in header
add_action('wp_head', 'wp_forecast_css');

?>